<?php

namespace App\Http\Controllers\Admin;

use App\Http\Requests\BuyNumberRequest;
use Backpack\CRUD\app\Http\Controllers\CrudController;
use Backpack\CRUD\app\Library\CrudPanel\CrudPanelFacade as CRUD;
use Twilio\Rest\Client;

/**
 * Class BuyNumberCrudController
 * @package App\Http\Controllers\Admin
 * @property-read \Backpack\CRUD\app\Library\CrudPanel\CrudPanel $crud
 */
class BuyNumberCrudController extends CrudController
{
    use \Backpack\CRUD\app\Http\Controllers\Operations\ListOperation;
    use \Backpack\CRUD\app\Http\Controllers\Operations\CreateOperation;
    use \Backpack\CRUD\app\Http\Controllers\Operations\InlineCreateOperation;

  //  use \Backpack\CRUD\app\Http\Controllers\Operations\UpdateOperation;
  //  use \Backpack\CRUD\app\Http\Controllers\Operations\DeleteOperation;
  //  use \Backpack\CRUD\app\Http\Controllers\Operations\ShowOperation;
    protected $_twilioClient;
    public function __construct(Client $twilioClient)
    {
        parent::__construct();
        $this->_twilioClient = $twilioClient;
    }
    /**
     * Configure the CrudPanel object. Apply settings to all operations.
     *
     * @return void
     */
    public function setup()
    {
      CRUD::setModel(\App\Models\TrackingNumber::class);

        CRUD::setRoute(config('backpack.base.route_prefix') . '/buy-number');
        CRUD::setEntityNameStrings('buy number', 'buy numbers');
        $numbers = $this->_twilioClient->availablePhoneNumbers("US")->local
            ->read(
                ['areaCode' => '850'],20
            );
        $this->data['numbers'] = $numbers;
        $this->crud->setListView('backpack::crud.buyNumberList');

    }


    /**
     * Define what happens when the List operation is loaded.
     *
     * @see  https://backpackforlaravel.com/docs/crud-operation-list-entries
     * @return void
     */
    protected function setupListOperation()
    {

        //  print_r($numbers);
        //CRUD::addColumn('type');
        //CRUD::column('tracking_numbers');
        CRUD::column('tracking_number');
                CRUD::column('name');
              /*  $test = $this->crud->getEntries();
                foreach($test as $t){
                  print_r($t->getAttributes());

                }*/

        /**
         * Columns can be defined using the fluent syntax or array syntax:
         * - CRUD::column('price')->type('number');
         * - CRUD::addColumn(['name' => 'price', 'type' => 'number']);
         */
    }

    /**
     * Define what happens when the Create operation is loaded.
     *
     * @see https://backpackforlaravel.com/docs/crud-operation-create
     * @return void
     */
    protected function setupCreateOperation()
    {
        CRUD::setValidation(BuyNumberRequest::class);

        CRUD::field('tracking_number');

    


        CRUD::field('name');
        CRUD::addField([
          'name' => 'type',
          'type'  => 'select_from_array',
          'options' => ['local'=>'Local','tollfree'=>'Toll-Free']
        ]);

        /**
         * Fields can be defined using the fluent syntax or array syntax:
         * - CRUD::field('price')->type('number');
         * - CRUD::addField(['name' => 'price', 'type' => 'number']));
         */
         CRUD::addField([
           // 1-n relationship
           'label'     => 'Client', // Table column heading
           'type'      => 'select',
           'name'      => 'client_id', // the column that contains the ID of that connected entity;
           'entity'    => 'client', // the method that defines the relationship in your Model
           'attribute' => 'name', // foreign key attribute that is shown to user
           'model'     => "App\Models\Client", // foreign key model
        'options'   => (function ($query) {
        return $query->orderBy('name', 'ASC')->get();
      }),
    ]);
    }

    /**
     * Define what happens when the Update operation is loaded.
     *
     * @see https://backpackforlaravel.com/docs/crud-operation-update
     * @return void
     */
    protected function setupUpdateOperation()
    {
        $this->setupCreateOperation();
    }
}
