<?php

namespace App\Http\Controllers\Admin;

use App\Http\Requests\CallerRequest;
use Backpack\CRUD\app\Http\Controllers\CrudController;
use Backpack\CRUD\app\Library\CrudPanel\CrudPanelFacade as CRUD;
use Backpack\CRUD\app\Library\Widget;
use App\Models\Caller;

/**
 * Class CallerCrudController
 * @package App\Http\Controllers\Admin
 * @property-read \Backpack\CRUD\app\Library\CrudPanel\CrudPanel $crud
 */
class CallerCrudController extends CrudController
{
    use \Backpack\CRUD\app\Http\Controllers\Operations\ListOperation;
    use \Backpack\CRUD\app\Http\Controllers\Operations\CreateOperation;
    use \Backpack\CRUD\app\Http\Controllers\Operations\UpdateOperation;
    use \Backpack\CRUD\app\Http\Controllers\Operations\DeleteOperation;
    use \Backpack\CRUD\app\Http\Controllers\Operations\ShowOperation;

    /**
     * Configure the CrudPanel object. Apply settings to all operations.
     * 
     * @return void
     */
    public function setup()
    {
        CRUD::setModel(\App\Models\Caller::class);
        CRUD::setRoute(config('backpack.base.route_prefix') . '/caller');
        CRUD::setEntityNameStrings('caller', 'callers');
    }

    /**
     * Define what happens when the List operation is loaded.
     * 
     * @see  https://backpackforlaravel.com/docs/crud-operation-list-entries
     * @return void
     */
    protected function setupListOperation()
    {   
        $lastMonthCount = Caller::whereBetween('created_at', [date('Y-m-d 00:00:00', strtotime("first day of previous month")), date('Y-m-d 23:59:59', strtotime("last day of previous month"))])->distinct('number')->count();
        $currentMonthCount = Caller::where('created_at', '>=', date('Y-m-01 00:00:00'))->distinct('number')->count();
        
        
        Widget::add()->to('before_content')->type('div')->class('row')->content([[
                    'type'       => 'card',
                    'wrapper' => ['class' => 'col-sm-6 col-md-3'], // optional
                    'class'   => 'card bg-dark text-white', // optional
                    'content'    => [
                        'header' => $lastMonthCount, // optional
                        'body'   => 'New Callers Last Month',
                    ],
                ], [
                    'type'       => 'card',
                    'wrapper' => ['class' => 'col-sm-6 col-md-3'],
                    'class'   => 'card bg-dark text-white', 
                    'content'    => [
                        'header' => $currentMonthCount, // optional
                        'body'   => 'New Callers This Month',
                    ],
                ]]);
        
        
        

        
        CRUD::column('number');
        CRUD::column('name');
        CRUD::column('state');
        CRUD::column('city');

    }

    /**
     * Define what happens when the Create operation is loaded.
     * 
     * @see https://backpackforlaravel.com/docs/crud-operation-create
     * @return void
     */
    protected function setupCreateOperation()
    {
        CRUD::setValidation(CallerRequest::class);

        CRUD::field('number');
        CRUD::field('name');
        CRUD::field('state');
        CRUD::field('city');

        /**
         * Fields can be defined using the fluent syntax or array syntax:
         * - CRUD::field('price')->type('number');
         * - CRUD::addField(['name' => 'price', 'type' => 'number'])); 
         */
    }

    /**
     * Define what happens when the Update operation is loaded.
     * 
     * @see https://backpackforlaravel.com/docs/crud-operation-update
     * @return void
     */
    protected function setupUpdateOperation()
    {
        $this->setupCreateOperation();
    }
}
