<?php

namespace App\Http\Controllers\Api;

use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{
  /**
   * Login The User
   * @param Request $request
   * @return User
   */
  public function loginUser(Request $request)
  {
      try {
          $validateUser = Validator::make($request->all(),
          [
              'email' => 'required|email',
              'password' => 'required'
          ]);

          if($validateUser->fails()){
              return response()->json([
                  'status' => false,
                  'message' => 'validation error',
                  'errors' => $validateUser->errors()
              ], 401);
          }

          if(!Auth::attempt($request->only(['email', 'password']))){
              return response()->json([
                  'status' => false,
                  'message' => 'Email & Password does not match with our record.',
              ], 401);
          }

          $user = User::where('email', $request->email)->first();

          return response()->json([
              'status' => true,
              'message' => 'User Logged In Successfully',
              'token' => $user->createToken("api-token")->plainTextToken
          ], 200);

      } catch (\Throwable $th) {
          return response()->json([
              'status' => false,
              'message' => $th->getMessage()
          ], 500);
      }
  }
}
